package org.etsi.osl.tmf.pm628.reposervices;

import jakarta.validation.Valid;
import org.etsi.osl.tmf.pm628.model.MeasurementCollectionJob;
import org.etsi.osl.tmf.pm628.repo.MeasurementCollectionJobRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
@Transactional
public class MeasurementCollectionJobService {

    private static final Logger log = LoggerFactory.getLogger(MeasurementCollectionJobService.class);

    private final MeasurementCollectionJobRepository measurementCollectionJobRepository;

    @Autowired
    public MeasurementCollectionJobService(MeasurementCollectionJobRepository measurementCollectionJobRepository) {
        this.measurementCollectionJobRepository = measurementCollectionJobRepository;
    }
    
    public List<MeasurementCollectionJob> findAllMeasurementCollectionJobs(){
        log.info("MeasurementCollectionJobService: LIST");
        return (List<MeasurementCollectionJob>) measurementCollectionJobRepository.findAll();
    }

    public MeasurementCollectionJob findMeasurementCollectionJobByUuid(String uuid){
        log.debug("MeasurementCollectionJob FIND BY UUID");
        Optional<MeasurementCollectionJob> measurementCollectionJob = measurementCollectionJobRepository.findByUuid(uuid);
        return measurementCollectionJob.orElse(null);
    }

    public MeasurementCollectionJob createMeasurementCollectionJob(MeasurementCollectionJob measurementCollectionJob){
        log.info("MeasurementCollectionJob CREATE: {}", measurementCollectionJob);
        return measurementCollectionJobRepository.save(measurementCollectionJob);
    }

    public MeasurementCollectionJob updateMeasurementCollectionJob(String uuid, @Valid MeasurementCollectionJob measurementCollectionJobUpdate){
        log.info("MeasurementCollectionJob UPDATE with UUID: {}", uuid);
        MeasurementCollectionJob measurementCollectionJob = measurementCollectionJobRepository.findByUuid(uuid).
                orElseThrow(() -> new IllegalArgumentException("No MeasurementCollectionJob with UUID: " + uuid));
        // Assuming you have a similar method for updating MeasurementCollectionJob fields
        measurementCollectionJob = updateMeasurementCollectionJobFields(measurementCollectionJob, measurementCollectionJobUpdate);
        measurementCollectionJobRepository.save(measurementCollectionJob);
        return measurementCollectionJob;
    }

    public Void deleteMeasurementCollectionJob(String uuid){
        log.info("MeasurementCollectionJob DELETE with UUID:{}", uuid);
        MeasurementCollectionJob measurementCollectionJob = measurementCollectionJobRepository.findByUuid(uuid)
                .orElseThrow(() -> new IllegalArgumentException("No MeasurementCollectionJob with UUID: " + uuid));
        measurementCollectionJobRepository.delete(measurementCollectionJob);

        return null;
    }

    private MeasurementCollectionJob updateMeasurementCollectionJobFields(MeasurementCollectionJob mcjCurrent, MeasurementCollectionJob mcjUpdate) {
        if (mcjUpdate.getJobOnDemand() != null){
            mcjCurrent.setJobOnDemand(
                    mcjUpdate.getJobOnDemand()
            );
        }
        if (mcjUpdate.getJobCollectionFilter() != null){
            mcjCurrent.setJobCollectionFilter(
                    mcjUpdate.getJobCollectionFilter()
            );
        }
        if (mcjUpdate.getJobId() != null){
            mcjCurrent.setJobId(
                    mcjUpdate.getJobId()
            );

        }
        if (mcjUpdate.getOutputFormat() != null){
            mcjCurrent.setOutputFormat(
                mcjUpdate.getOutputFormat()
            );
        }
        if (mcjUpdate.getAtBaseType() != null){
            mcjCurrent.setAtBaseType(
                    mcjUpdate.getAtBaseType()
            );
        }
        if (mcjUpdate.getReportingPeriod() != null){
            mcjCurrent.setReportingPeriod(
                    mcjUpdate.getReportingPeriod()
            );
        }
        if (mcjUpdate.getSearchTaskFilter() != null){
            mcjCurrent.setSearchTaskFilter(
                    mcjUpdate.getSearchTaskFilter()
            );
        }
        if (mcjUpdate.getAdminState() != null){
            mcjCurrent.setAdminState(
                    mcjUpdate.getAdminState()
            );
        }
        if (mcjUpdate.getAtSchemaLocation() != null){
            mcjCurrent.setAtSchemaLocation(
                    mcjUpdate.getAtSchemaLocation()
            );
        }
        if (mcjUpdate.getAtType() != null){
            mcjCurrent.setAtType(
                    mcjUpdate.getAtType()
            );
        }
        if (mcjUpdate.getConsumingApplicationId() != null){
            mcjCurrent.setConsumingApplicationId(
                    mcjUpdate.getConsumingApplicationId()
            );
        }
        if (mcjUpdate.getDataAccessEndpoint() != null){
            mcjCurrent.setDataAccessEndpoint(
                    mcjUpdate.getDataAccessEndpoint()
            );
        }
        if (mcjUpdate.getExecutionState() != null){
            mcjCurrent.setExecutionState(
                    mcjUpdate.getExecutionState()
            );
        }
        if (mcjUpdate.getFileTransferData() != null){
            mcjCurrent.setFileTransferData(
                    mcjUpdate.getFileTransferData()
            );
        }
        if (mcjUpdate.getGranularity() != null){
            mcjCurrent.setGranularity(
                    mcjUpdate.getGranularity()
            );
        }
        if (mcjUpdate.getHref() != null){
            mcjCurrent.setHref(
                    mcjUpdate.getHref()
            );
        }
        if (mcjUpdate.getId() != null){
            mcjCurrent.setId(
                    mcjUpdate.getId()
            );
        }
        if (mcjUpdate.getJobPriority() != null){
            mcjCurrent.setJobPriority(
                    mcjUpdate.getJobPriority()
            );
        }
        if (mcjUpdate.getMonitoredClassCriteria() != null){
            mcjCurrent.setMonitoredClassCriteria(
                    mcjUpdate.getMonitoredClassCriteria()
            );
        }
        if (mcjUpdate.getMonitoredInstancesCriteria() != null){
            mcjCurrent.setMonitoredInstancesCriteria(
                    mcjUpdate.getMonitoredInstancesCriteria()
            );
        }
        if (mcjUpdate.getPerformanceIndicatorGroupSpecification() != null){
            mcjCurrent.setPerformanceIndicatorGroupSpecification(
                    mcjUpdate.getPerformanceIndicatorGroupSpecification()
            );
        }
        if (mcjUpdate.getPerformanceIndicatorSpecification() != null){
            mcjCurrent.setPerformanceIndicatorSpecification(
                    mcjUpdate.getPerformanceIndicatorSpecification()
            );
        }
        if (mcjUpdate.getProducingApplicationId() != null){
            mcjCurrent.setProducingApplicationId(
                    mcjUpdate.getProducingApplicationId()
            );
        }
        if (mcjUpdate.getScheduleDefinition() != null){
            mcjCurrent.setScheduleDefinition(
                    mcjUpdate.getScheduleDefinition()
            );
        }
        if (mcjUpdate.getTrackingRecord() != null){
            mcjCurrent.setTrackingRecord(
                    mcjUpdate.getTrackingRecord()
            );
        }

        return mcjCurrent;
    }
}
